import { IFriend, IParams, IServerLikesParams } from './methods';
export interface IRequestOptions {
    group: Group;
    target?: string;
    prop: string;
    params?: any;
}
export declare enum Group {
    PROFILE = "profile",
    SERVER = "server"
}
/**
 * Make requests to NameMC API
 * @example
 * Get player friends
 * ```
 * const api = new API();
 *
 * api.profile.friends({
 *     target: '2e9d9da1-97e9-4564-890b-6f056c4e372f'
 * });
 * ```
 */
export declare class API {
    static ENDPOINT: string;
    /**
     * Profile API section methods
     */
    readonly profile: {
        /**
         * Get player friends
         */
        friends(params: IParams): Promise<IFriend[]>;
    };
    /**
     * Server API section methods
     */
    readonly server: {
        /**
         * Check player server like
         */
        likes(params: IParams): Promise<string[]>;
        /**
         * Get all players list server likes
         */
        likes(params: IServerLikesParams): Promise<boolean>;
    };
    /**
     * @hidden
     */
    private worker;
    /**
     * @hidden
     */
    constructor();
    /**
     * @hidden
     */
    get [Symbol.toStringTag](): string;
    /**
     * @hidden
     */
    protected request({ group, target, prop, params }: IRequestOptions): Promise<any>;
}
export * from './methods';
