"use strict";

var _exportNames = {
  Group: true,
  API: true
};
exports.Group = exports.API = void 0;

var _axios = require("axios");

var packageMeta = require("../../package.json");

var _methods = require("./methods");

Object.keys(_methods).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _methods[key]) return;
  exports[key] = _methods[key];
});
// @ts-ignore
const {
  version,
  homepage
} = packageMeta;
var Group;
exports.Group = Group;

(function (Group) {
  Group["PROFILE"] = "profile";
  Group["SERVER"] = "server";
})(Group || (exports.Group = Group = {}));
/**
 * Make requests to NameMC API
 * @example
 * Get player friends
 * ```
 * const api = new API();
 *
 * api.profile.friends({
 *     target: '2e9d9da1-97e9-4564-890b-6f056c4e372f'
 * });
 * ```
 */


class API {
  /**
   * @hidden
   */
  constructor() {
    this.worker = _axios.default.create({
      baseURL: API.ENDPOINT,
      headers: typeof window === 'undefined' ? {
        'User-Agent': `NameMCWrapper/${version} (${homepage})`
      } : {}
    });

    for (const group of Object.values(Group)) {
      this[group] = new Proxy(Object.create(null), {
        get: (object, prop) => ({
          target,
          ...params
        }) => this.request({
          group,
          target,
          prop,
          params
        })
      });
    }
  }
  /**
   * @hidden
   */


  get [Symbol.toStringTag]() {
    return this.constructor.name;
  }
  /**
   * @hidden
   */


  request({
    group,
    target,
    prop,
    params
  }) {
    return this.worker.get(`/${group}/${target ? `${target}/` : ''}${prop}`, {
      params
    }).then(({
      data
    }) => data);
  }

}

exports.API = API;
API.ENDPOINT = 'https://api.namemc.com';