import { Context, IContextOptions } from '../context';
import { Hash } from '../player';
import { kSerializeData } from '../../utils';
import { CapeName, CapeType } from './types';
export interface ICapeContextOptions extends IContextOptions<CapeContext> {
    /**
     * Cape hash
     */
    hash: Hash;
}
export declare class CapeContext extends Context<CapeContext> {
    /**
     * Cape hash
     */
    readonly hash: Hash;
    /**
     * @hidden
     */
    protected endpoint: string;
    /**
     * @hidden
     */
    constructor({ hash, ...options }: ICapeContextOptions);
    /**
     * Get cape url
     */
    get url(): string;
    /**
     * Get cape type
     */
    get type(): CapeType;
    /**
     * Get cape name
     */
    get name(): CapeName;
    /**
     * Check is Minecraft cape
     */
    get isMinecraft(): boolean;
    /**
     * Check is Optifine cape
     */
    get isOptifine(): boolean;
    /**
     * @hidden
     */
    [kSerializeData](): any;
}
export * from './types';
