"use strict";

var _exportNames = {
  CapeContext: true
};
exports.CapeContext = void 0;

var _context = require("../context");

var _utils = require("../../utils");

var _types = require("./types");

Object.keys(_types).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _types[key]) return;
  exports[key] = _types[key];
});

class CapeContext extends _context.Context {
  /**
   * @hidden
   */
  constructor({
    hash,
    ...options
  }) {
    super(options);
    /**
     * @hidden
     */

    this.endpoint = this.options.getEndpoint();
    this.hash = hash;
  }
  /**
   * Get cape url
   */


  get url() {
    return `${this.endpoint}/texture/${this.hash}.png`;
  }
  /**
   * Get cape type
   */


  get type() {
    return Object.values(_types.CapeHash).includes(this.hash) ? _types.CapeType.MINECRAFT : _types.CapeType.OPTIFINE;
  }
  /**
   * Get cape name
   */


  get name() {
    const cape = new Map(Object.entries(_types.CapeHash).map(entry => entry.reverse())).get(this.hash);

    if (cape) {
      const capeName = new Map(Object.entries(_types.CapeName)).get(cape);
      return capeName;
    }

    return _types.CapeName.OPTIFINE;
  }
  /**
   * Check is Minecraft cape
   */


  get isMinecraft() {
    return this.type === _types.CapeType.MINECRAFT;
  }
  /**
   * Check is Optifine cape
   */


  get isOptifine() {
    return this.type === _types.CapeType.OPTIFINE;
  }
  /**
   * @hidden
   */


  [_utils.kSerializeData]() {
    return (0, _utils.pickProperties)(this, ['hash', 'url', 'name', 'type']);
  }

}

exports.CapeContext = CapeContext;