import { AxiosInstance } from 'axios';
import { Options } from '../options';
import { API } from '../api';
import { SkinContext } from './skin';
import { RendersContext } from './renders';
import { ServerContext } from './server';
import { SearchContext } from './search';
import { PlayerContext } from './player';
import { CapeContext } from './cape';
import { kSerializeData } from '../utils';
export interface IContextOptions<P = any | void> {
    options: Options;
    client: AxiosInstance;
    api: API;
    payload?: {
        [K in keyof P]?: P[keyof P];
    };
}
export declare type ContextUnion = SkinContext | CapeContext | RendersContext | PlayerContext | ServerContext | SearchContext;
/**
 * @hidden
 */
export declare class Context<T extends ContextUnion> {
    /**
     * Parameters passed to the constructor during NameMC class initialization
     *
     * @hidden
     */
    readonly options: IContextOptions['options'];
    /**
     * Class for API Requests
     *
     * @hidden
     */
    readonly api: IContextOptions['api'];
    /**
     * Axios Client for requests
     *
     * @hidden
     */
    readonly client: IContextOptions['client'];
    /**
     * @hidden
     */
    payload: Record<string, any>;
    /**
     * @hidden
     */
    constructor({ options, client, api, payload }: IContextOptions);
    /**
     * @hidden
     */
    protected setupPayload(): void;
    /**
     * Get context JSON
     */
    toJSON(): T;
    /**
     * @hidden
     */
    [kSerializeData](): any;
}
