import { Element } from 'cheerio';
import { Context, SkinContext, CapeContext, ServerContext, IContextOptions } from '../';
import { kSerializeData } from '../../utils';
import { IGetFollowersOptions, IUsername } from './types';
import { IFriend } from '../../api';
export interface IPlayerContextOptions extends IContextOptions {
    data?: string | Element | Element[];
    isSearch?: boolean;
}
export declare class PlayerContext extends Context<PlayerContext> {
    #private;
    /**
     * Profile ID
     */
    readonly id: number;
    /**
     * Player uuid
     */
    readonly uuid: string;
    /**
     * Player username
     */
    readonly username: string;
    /**
     * Profile views
     */
    readonly views: number;
    /**
     * Nickname history
     */
    readonly names: IUsername[];
    /**
     * Skin history
     */
    readonly skins: SkinContext[];
    /**
     * Player capes
     */
    readonly capes: CapeContext[];
    /**
     * Player friends
     */
    readonly friends: IFriend[];
    /**
     * Player favorite servers
     */
    readonly servers: ServerContext[];
    /**
     * Player followers
     */
    readonly followers: PlayerContext[];
    /**
     * Player following
     */
    readonly following: PlayerContext[];
    /**
     * Following date, available when receiving player followers/following
     */
    readonly followingDate: number;
    /**
     * @hidden
     */
    constructor({ data, isSearch, ...options }: IPlayerContextOptions);
    /**
     * Check payload loaded
     */
    get isExtended(): boolean;
    /**
     * Get clean UUID
     */
    get cleanUUID(): string;
    /**
     * Load all profile information
     */
    loadPayload(): Promise<void>;
    /**
     * Get player followers
     */
    getFollowers(options: IGetFollowersOptions): Promise<PlayerContext['followers']>;
    /**
     * Get player following
     */
    getFollowing(options: IGetFollowersOptions): Promise<PlayerContext['followers']>;
    /**
     * @hidden
     */
    [kSerializeData](): any;
}
export * from './types';
