"use strict";

var _exportNames = {
  PlayerContext: true
};
exports.PlayerContext = void 0;

var _cheerio = require("cheerio");

var _error = require("../../error");

var _ = require("../");

var _utils = require("../../utils");

var _types = require("./types");

Object.keys(_types).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _types[key]) return;
  exports[key] = _types[key];
});

var __classPrivateFieldGet = void 0 && (void 0).__classPrivateFieldGet || function (receiver, state, kind, f) {
  if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
  if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
  return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};

var __classPrivateFieldSet = void 0 && (void 0).__classPrivateFieldSet || function (receiver, state, value, kind, f) {
  if (kind === "m") throw new TypeError("Private method is not writable");
  if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
  if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
  return kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value), value;
};

var _PlayerContext_instances, _PlayerContext_extended, _PlayerContext_loadFollowers, _PlayerContext_parseFollowers, _PlayerContext_parseUsernameHistory, _PlayerContext_parseServers, _PlayerContext_parseCapes, _PlayerContext_parseSkins, _PlayerContext_parseInfoColumns, _PlayerContext_parseId;

class PlayerContext extends _.Context {
  /**
   * @hidden
   */
  constructor({
    data,
    isSearch,
    ...options
  }) {
    super(options);

    _PlayerContext_instances.add(this);
    /**
     * Profile ID
     */


    this.id = 0;
    /**
     * Player uuid
     */

    this.uuid = '';
    /**
     * Player username
     */

    this.username = '';
    /**
     * Profile views
     */

    this.views = 0;
    /**
     * Nickname history
     */

    this.names = [];
    /**
     * Skin history
     */

    this.skins = [];
    /**
     * Player capes
     */

    this.capes = [];
    /**
     * Player friends
     */

    this.friends = [];
    /**
     * Player favorite servers
     */

    this.servers = [];
    /**
     * Player followers
     */

    this.followers = [];
    /**
     * Player following
     */

    this.following = [];
    /**
     * Following date, available when receiving player followers/following
     */

    this.followingDate = 0;
    /**
     * Payload loaded
     * @hidden
     */

    _PlayerContext_extended.set(this, false);

    this.setupPayload();

    if (!data) {
      return;
    }

    const $ = _cheerio.default.load(data);

    this.skins = __classPrivateFieldGet(this, _PlayerContext_instances, "m", _PlayerContext_parseSkins).call(this, $, isSearch);

    if (isSearch) {
      this.names = __classPrivateFieldGet(this, _PlayerContext_instances, "m", _PlayerContext_parseUsernameHistory).call(this, $('tr'));
    } else {
      this.capes = __classPrivateFieldGet(this, _PlayerContext_instances, "m", _PlayerContext_parseCapes).call(this, $);
      this.servers = __classPrivateFieldGet(this, _PlayerContext_instances, "m", _PlayerContext_parseServers).call(this, $);

      const {
        uuid,
        views,
        names
      } = __classPrivateFieldGet(this, _PlayerContext_instances, "m", _PlayerContext_parseInfoColumns).call(this, $);

      this.uuid = uuid;
      this.views = views;
      this.names = names;
    }

    this.username = this.names[this.names.length - 1].username;
    this.id = __classPrivateFieldGet(this, _PlayerContext_instances, "m", _PlayerContext_parseId).call(this, $, isSearch);
  }
  /**
   * Check payload loaded
   */


  get isExtended() {
    return __classPrivateFieldGet(this, _PlayerContext_extended, "f");
  }
  /**
   * Get clean UUID
   */


  get cleanUUID() {
    return this.uuid.replace(/-/g, '');
  }
  /**
   * Load all profile information
   */


  async loadPayload() {
    if (this.isExtended) {
      return;
    }

    if (!(this.username && this.uuid && this.id)) {
      const username = this.uuid || this.username;

      if (!username.match(_utils.nameRegExp)) {
        throw new _error.WrapperError('INVALID_NICKNAME', username);
      }

      await this.client.get(`/profile/${username}`).then(({
        request,
        headers,
        data
      }) => {
        var _request$res;

        if ((headers['x-final-url'] || (request === null || request === void 0 ? void 0 : (_request$res = request.res) === null || _request$res === void 0 ? void 0 : _request$res.responseUrl) || request.responseURL).match(_utils.profileRegExp)) {
          this.payload = new PlayerContext({
            data,
            ...this
          }).toJSON();
          return this.setupPayload();
        }

        throw new _error.WrapperError('NOT_FOUND', username);
      });
    }

    this.payload = await this.api.profile.friends({
      target: this.uuid
    }).then(friends => ({
      friends
    }));
    this.setupPayload();

    __classPrivateFieldSet(this, _PlayerContext_extended, true, "f");
  }
  /**
   * Get player followers
   */


  getFollowers(options) {
    return __classPrivateFieldGet(this, _PlayerContext_instances, "m", _PlayerContext_loadFollowers).call(this, { ...options,
      section: _types.FollowersSection.FOLLOWERS
    });
  }
  /**
   * Get player following
   */


  getFollowing(options) {
    return __classPrivateFieldGet(this, _PlayerContext_instances, "m", _PlayerContext_loadFollowers).call(this, { ...options,
      section: _types.FollowersSection.FOLLOWING
    });
  }
  /**
   * @hidden
   */


  [(_PlayerContext_extended = new WeakMap(), _PlayerContext_instances = new WeakSet(), _PlayerContext_loadFollowers =
  /**
   * @hidden
   */
  async function _PlayerContext_loadFollowers({
    section,
    page = 1,
    sort = {}
  }) {
    if (!__classPrivateFieldGet(this, _PlayerContext_extended, "f")) {
      await this.loadPayload();
    }

    const filter = Object.entries(sort).map(filter => filter.join(':')).join(',');
    const loadedFollowers = await this.client.get(`/profile/${this.uuid}/${section}`, {
      params: {
        page,
        sort: filter
      }
    }).then(({
      data
    }) => __classPrivateFieldGet(this, _PlayerContext_instances, "m", _PlayerContext_parseFollowers).call(this, data));
    const followers = loadedFollowers.reduce((followers, follower) => {
      if (!followers.filter(({
        username
      }) => username === follower.username).length) {
        followers.push(follower);
      }

      return followers;
    }, this[section]);
    this.payload = {
      [section]: followers
    };
    this.setupPayload();
    return loadedFollowers;
  }, _PlayerContext_parseFollowers = function _PlayerContext_parseFollowers(data) {
    const $ = _cheerio.default.load(data);

    const uuid = $('tbody > input[name=\'profile\']').map((index, {
      attribs: {
        value: uuid
      }
    }) => uuid).get();
    const names = $('tbody > tr').map((index, element) => {
      const $ = _cheerio.default.load(element.children);

      const children = $('td');

      const name = _cheerio.default.load(children.get(2));

      const username = name.text(); // @ts-ignore

      const {
        attribs: {
          datetime
        }
      } = children.get(3).firstChild;
      return {
        username,
        date: new Date(datetime).getTime()
      };
    }).get();
    return names.map(({
      username,
      date: followingDate
    }, index) => new PlayerContext({ ...this,
      payload: {
        username,
        followingDate,
        uuid: uuid[index]
      }
    }));
  }, _PlayerContext_parseUsernameHistory = function _PlayerContext_parseUsernameHistory(element) {
    return element.map((index, element) => {
      const $ = _cheerio.default.load(element);

      const name = $('a').get(0);
      const time = $('time').get(0);

      if (name) {
        var _time$attribs;

        // @ts-ignore
        const {
          children: [{
            data: username
          }]
        } = name;
        const changed_at = (time === null || time === void 0 ? void 0 : (_time$attribs = time.attribs) === null || _time$attribs === void 0 ? void 0 : _time$attribs.datetime) || null;
        return {
          username,
          changed_at,
          timestamp: changed_at ? new Date(changed_at).getTime() : null
        };
      }
    }).get().reverse();
  }, _PlayerContext_parseServers = function _PlayerContext_parseServers($) {
    return $('a > img.server-icon').map((index, element) => {
      var _element$parent, _element$parent$attri, _element$next;

      const ip = (_element$parent = element.parent) === null || _element$parent === void 0 ? void 0 : (_element$parent$attri = _element$parent.attribs) === null || _element$parent$attri === void 0 ? void 0 : _element$parent$attri.href.replace(_utils.serverRegExp, '$1');
      const title = ((_element$next = element.next) === null || _element$next === void 0 ? void 0 : _element$next.data) || '';
      const {
        attribs: {
          src: icon
        }
      } = element;
      return new _.ServerContext({ ...this,
        extended: false,
        payload: {
          ip,
          title,
          icon
        }
      });
    }).get();
  }, _PlayerContext_parseCapes = function _PlayerContext_parseCapes($) {
    return $('canvas.cape-2d').map((index, {
      attribs: {
        'data-cape': hash
      }
    }) => new _.CapeContext({ ...this,
      hash
    })).get();
  }, _PlayerContext_parseSkins = function _PlayerContext_parseSkins($, isSearch) {
    return $(isSearch ? 'img.skin-2d' : 'canvas.skin-2d.skin-button').map((index, {
      attribs: {
        'data-id': hash,
        'data-model': model = _.Model.UNKNOWN,
        title,
        src
      }
    }) => new _.SkinContext({ ...this,
      payload: isSearch ? _.SkinContext.parseSkinLink(src) : {
        model,
        id: hash,
        createdAt: (0, _utils.convertDate)(title)
      }
    })).get();
  }, _PlayerContext_parseInfoColumns = function _PlayerContext_parseInfoColumns($) {
    const [baseInfoRaw, usernameHistoryRaw] = $('div.card.mb-3 > div.card-body').map((index, element) => {
      const $ = _cheerio.default.load(element);

      const body = $('div.card-body');

      switch (index) {
        case 0:
          return body.children('div.row.no-gutters');

        case 1:
          return body.find('tr:not([class])').map((index, element) => {
            element.children.push(element.next);
            return element;
          });
      }
    }).get();
    const baseInfo = baseInfoRaw.map((index, element) => {
      const $ = _cheerio.default.load(element);

      switch (index) {
        case 0:
        case 1:
          return $('div.col-12 > samp').text();

        case 2:
          {
            const views = parseInt($('div.col-auto').text());
            return Number(views);
          }
      }
    }).get();

    const names = __classPrivateFieldGet(this, _PlayerContext_instances, "m", _PlayerContext_parseUsernameHistory).call(this, usernameHistoryRaw);

    const [uuid,, views] = baseInfo;
    return {
      uuid,
      views,
      names
    };
  }, _PlayerContext_parseId = function _PlayerContext_parseId($, isSearch) {
    if (isSearch) {
      const {
        attribs: {
          href
        }
      } = $('div.card-header a').get(0);
      const match = href.match(_utils.profileRegExp);

      if (!match) {
        return 0;
      }

      const [,, id] = match;
      return Number(id) || 0;
    }

    const link = $(`a[href^="/profile/${this.username}"].nav-link`).get(0);

    if (link) {
      const {
        attribs: {
          href
        }
      } = link;
      return Number(href.split('.').pop()) || 0;
    }

    return 0;
  }, _utils.kSerializeData)]() {
    return (0, _utils.pickProperties)(this, ['id', 'uuid', 'username', 'views', 'names', 'skins', 'capes', 'friends', 'followers', 'following', 'followingDate', 'servers']);
  }

}

exports.PlayerContext = PlayerContext;