import { Context, IContextOptions } from '../context';
import { Model, ModelUnion } from '../skin';
import { Hash } from '../player';
import { kSerializeData } from '../../utils';
declare type RenderOptions = Pick<RendersContext, 'skin'> & Partial<Omit<RendersContext, 'skin' | keyof IContextOptions>>;
export interface IRendersContextOptions extends IContextOptions<RenderOptions>, RenderOptions {
}
export declare class RendersContext extends Context<RendersContext> {
    /**
     * Skin id
     */
    readonly skin: Hash;
    /**
     * Cape hash
     */
    readonly cape: Hash;
    /**
     * Skin model
     */
    readonly model: Model | ModelUnion;
    /**
     * Image render width
     */
    readonly width: number;
    /**
     * Image render height
     */
    readonly height: number;
    /**
     * Image render scale
     */
    readonly scale: number;
    /**
     * Show skin overlay
     */
    readonly overlay: boolean;
    /**
     * Model horizontal rotation angle
     */
    readonly theta: number;
    /**
     * Model vertical rotation angle
     */
    readonly phi: number;
    /**
     * Model animation time
     */
    readonly time: number;
    /**
     * Model HEX shadow color
     *
     * @see {@link https://www.color-hex.com/ | HEX Color generator}
     */
    readonly shadow_color: string;
    /**
     * Model shadow color
     */
    readonly shadow_radius: number;
    /**
     * Model horizontal shadow offset
     */
    readonly shadow_x: number;
    /**
     * Model vertical shadow offset
     */
    readonly shadow_y: number;
    /**
     * @hidden
     */
    protected endpoint: string;
    /**
     * NameMC renders domain
     */
    static DOMAIN: string;
    /**
     * @hidden
     */
    constructor({ options, client, payload, api, ...rendersOptions }: IRendersContextOptions);
    /**
     * Get body render models
     */
    get body(): ({
        /**
         * Body front render model url
         */
        front: string;
        /**
         * Body front & back render model url
         */
        front_and_back: string;
    });
    /**
     * Get face render model url
     */
    get face(): string;
    [kSerializeData](): any;
}
export {};
