"use strict";

exports.RendersContext = void 0;

var _context = require("../context");

var _skin = require("../skin");

var _utils = require("../../utils");

class RendersContext extends _context.Context {
  /**
   * @hidden
   */
  constructor({
    options,
    client,
    payload,
    api,
    ...rendersOptions
  }) {
    super({
      options,
      client,
      payload,
      api
    });
    /**
     * Skin id
     */

    this.skin = _utils.STEVE_SKIN_HASH;
    /**
     * Cape hash
     */

    this.cape = '';
    /**
     * Skin model
     */

    this.model = _skin.Model.UNKNOWN;
    /**
     * Image render width
     */

    this.width = 600;
    /**
     * Image render height
     */

    this.height = 300;
    /**
     * Image render scale
     */

    this.scale = 4;
    /**
     * Show skin overlay
     */

    this.overlay = true;
    /**
     * Model horizontal rotation angle
     */

    this.theta = 30;
    /**
     * Model vertical rotation angle
     */

    this.phi = 20;
    /**
     * Model animation time
     */

    this.time = 90;
    /**
     * Model HEX shadow color
     *
     * @see {@link https://www.color-hex.com/ | HEX Color generator}
     */

    this.shadow_color = '000';
    /**
     * Model shadow color
     */

    this.shadow_radius = 0;
    /**
     * Model horizontal shadow offset
     */

    this.shadow_x = 0;
    /**
     * Model vertical shadow offset
     */

    this.shadow_y = 0;
    /**
     * @hidden
     */

    this.endpoint = this.options.getEndpoint({
      domain: RendersContext.DOMAIN
    });
    this.payload = { ...this.payload,
      ...rendersOptions
    };
    this.setupPayload();
  }
  /**
   * Get body render models
   */


  get body() {
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    const {
      endpoint,
      options,
      client,
      api,
      payload,
      ...params
    } = this;
    const front = `${endpoint}/3d/skin/body.png?${new URLSearchParams(params)}`;
    return {
      front,
      front_and_back: `${front}&front_and_back=true`
    };
  }
  /**
   * Get face render model url
   */


  get face() {
    const {
      endpoint,
      skin,
      overlay,
      scale
    } = this;
    return `${endpoint}/2d/skin/face.png?skin=${skin}&overlay=${overlay}&scale=${scale}`;
  }

  [_utils.kSerializeData]() {
    return (0, _utils.pickProperties)(this, ['body', 'face']);
  }

}
/**
 * NameMC renders domain
 */


exports.RendersContext = RendersContext;
RendersContext.DOMAIN = 's.namemc.com';