"use strict";

var _exportNames = {
  SearchContext: true
};
exports.SearchContext = void 0;

var _cheerio = require("cheerio");

var _context = require("../context");

var _skin = require("../skin");

var _player = require("../player");

var _server = require("../server");

var _utils = require("../../utils");

var _types = require("./types");

Object.keys(_types).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _types[key]) return;
  exports[key] = _types[key];
});

var __classPrivateFieldGet = void 0 && (void 0).__classPrivateFieldGet || function (receiver, state, kind, f) {
  if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
  if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
  return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};

var _SearchContext_instances, _SearchContext_parseStatusBar, _SearchContext_parseServers, _SearchContext_parsePlayers;

class SearchContext extends _context.Context {
  /**
   * @hidden
   */
  constructor({
    data,
    ...options
  }) {
    super(options);

    _SearchContext_instances.add(this);
    /**
     * Search Query
     */


    this.query = '';
    /**
     * Query name info
     */

    this.name = {
      /**
       * Name status
       */
      status: _types.NameStatus.INVALID,

      /**
       * Name availability timestamp
       */
      availabilityTime: null,

      /**
       * Name availability time
       */
      availabilityAt: null,

      /**
       * Name searches
       */
      searches: 0
    };
    /**
     * Query players
     */

    this.players = [];
    /**
     * Query servers
     */

    this.servers = [];
    /**
     * Query skin tag
     */

    this.skin = null;
    this.setupPayload();

    if (!data) {
      return;
    }

    const $ = _cheerio.default.load(data);

    const [skinTagCol, mainCol] = $('main > div > div.col-12').map((index, element) => _cheerio.default.load(element)).get();
    const hasMainCol = Boolean(mainCol);

    if (hasMainCol) {
      const skin = skinTagCol('div.card-body').get(0);

      if (skin) {
        this.skin = new _skin.SkinContext({ ...this,
          data: skin
        });
      }
    }

    (hasMainCol ? mainCol : skinTagCol)('div.card.mb-3, div:not(.ad-container).mb-3').get().forEach(element => {
      const $ = _cheerio.default.load(element);

      const hasStatusBar = __classPrivateFieldGet(this, _SearchContext_instances, "m", _SearchContext_parseStatusBar).call(this, $);

      if (hasStatusBar) {
        return;
      }

      const hasServers = __classPrivateFieldGet(this, _SearchContext_instances, "m", _SearchContext_parseServers).call(this, $);

      if (hasServers) {
        return;
      }

      __classPrivateFieldGet(this, _SearchContext_instances, "m", _SearchContext_parsePlayers).call(this, $);
    });
  }
  /**
   * Check is query name is available
   */


  get isAvailable() {
    return this.name.status === _types.NameStatus.AVAILABLE;
  }
  /**
   * Check is query name is available available later
   */


  get isAvailableLater() {
    return this.name.status === _types.NameStatus.AVAILABLE_LATER;
  }
  /**
   * Check is query name is unavailable
   */


  get isUnavailable() {
    return this.name.status === _types.NameStatus.UNAVAILABLE;
  }
  /**
   * Check is query name is invalid
   */


  get isInvalid() {
    return this.name.status === _types.NameStatus.INVALID;
  }
  /**
   * Check query has players
   */


  get hasPlayers() {
    return Boolean(this.players.length);
  }
  /**
   * Check query has servers
   */


  get hasServers() {
    return Boolean(this.servers.length);
  }
  /**
   * Check query has skin tag name
   */


  get hasSkinTag() {
    return Boolean(this.skin);
  }
  /**
   * @hidden
   */


  [(_SearchContext_instances = new WeakSet(), _utils.kSerializeData)]() {
    return (0, _utils.pickProperties)(this, ['query', 'name', 'players', 'servers', 'skin']);
  }

}

exports.SearchContext = SearchContext;
_SearchContext_parseStatusBar = function _SearchContext_parseStatusBar($) {
  const statusBar = $('div#status-bar');

  if (statusBar.length) {
    this.name.status = statusBar.hasClass('bg-success') ? _types.NameStatus.AVAILABLE : statusBar.hasClass('bg-info') ? _types.NameStatus.AVAILABLE_LATER : statusBar.hasClass('bg-warning') ? _types.NameStatus.UNAVAILABLE : _types.NameStatus.INVALID;
    const availabilityTime = statusBar.find('time');

    if (availabilityTime.length) {
      this.name.availabilityAt = Number(availabilityTime.attr().datetime);
      this.name.availabilityTime = new Date(this.name.availabilityAt).getTime();
    }

    this.name.searches = parseInt(statusBar.find('div.tabular').text());
    return true;
  }

  return false;
}, _SearchContext_parseServers = function _SearchContext_parseServers($) {
  const servers = $('div.table-responsive tr');

  if (servers.length) {
    this.servers.push(...servers.map((index, element) => new _server.ServerContext({ ...this,
      data: element,
      isSearch: true
    })).get());
    return true;
  }

  return false;
}, _SearchContext_parsePlayers = function _SearchContext_parsePlayers($) {
  const data = $('div.card').get(0);
  this.players.push(new _player.PlayerContext({ ...this,
    data,
    isSearch: true
  }));
};