import { Element } from 'cheerio';
import { Context, IContextOptions } from '../context';
import { kSerializeData } from '../../utils';
import { ICheckServerLikeOptions } from './types';
export interface IServerContextOptions extends IContextOptions {
    data?: string | Element | Element[];
    extended?: boolean;
    isSearch?: boolean;
}
export declare class ServerContext extends Context<ServerContext> {
    #private;
    /**
     * Server ip
     */
    readonly ip: string;
    /**
     * Server title
     */
    readonly title: string;
    /**
     * Server icon url
     */
    readonly icon: string;
    /**
     * Server motd
     */
    readonly motd: {
        clear: string;
        html: string;
    };
    /**
     * Server current players stats
     */
    readonly players: {
        /**
         * Online now
         */
        online: number;
        /**
         * Maximum online
         */
        max: number;
    };
    /**
     * Server country
     */
    readonly country: {
        /**
         * Country name
         */
        name: string;
        /**
         * Country SVG Emoji image
         */
        image: string;
        /**
         * Country Emoji
         */
        emoji: string;
    } | null;
    /**
     * Server rating
     */
    readonly rating: number;
    /**
     * Server version
     */
    readonly version: string | null;
    /**
     * Server uptime
     */
    readonly uptime: number | null;
    /**
     * @hidden
     */
    constructor({ data, extended, isSearch, ...options }: IServerContextOptions);
    /**
     * Check payload loaded
     */
    get isExtended(): boolean;
    /**
     * Check like status for player
     */
    checkLike(username: ICheckServerLikeOptions['username']): Promise<boolean>;
    /**
     * Load all server information
     */
    loadPayload(): Promise<void>;
    /**
     * @hidden
     */
    [kSerializeData](): any;
}
export * from './types';
