"use strict";

var _exportNames = {
  ServerContext: true
};
exports.ServerContext = void 0;

var _cheerio = require("cheerio");

var _context = require("../context");

var _error = require("../../error");

var _utils = require("../../utils");

var _types = require("./types");

Object.keys(_types).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _types[key]) return;
  exports[key] = _types[key];
});

var __classPrivateFieldGet = void 0 && (void 0).__classPrivateFieldGet || function (receiver, state, kind, f) {
  if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
  if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
  return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};

var __classPrivateFieldSet = void 0 && (void 0).__classPrivateFieldSet || function (receiver, state, value, kind, f) {
  if (kind === "m") throw new TypeError("Private method is not writable");
  if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
  if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
  return kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value), value;
};

var _ServerContext_instances, _ServerContext_extended, _ServerContext_parseServerCard, _ServerContext_parseServerCountry, _ServerContext_parseIP, _ServerContext_parseSearchTableRow;

class ServerContext extends _context.Context {
  /**
   * @hidden
   */
  constructor({
    data,
    extended,
    isSearch,
    ...options
  }) {
    super(options);

    _ServerContext_instances.add(this);
    /**
     * Server ip
     */


    this.ip = '';
    /**
     * Server title
     */

    this.title = '';
    /**
     * Server icon url
     */

    this.icon = '';
    /**
     * Server motd
     */

    this.motd = {
      clear: '',
      html: ''
    };
    /**
     * Server current players stats
     */

    this.players = {
      /**
       * Online now
       */
      online: 0,

      /**
       * Maximum online
       */
      max: 0
    };
    /**
     * Server country
     */

    this.country = null;
    /**
     * Server rating
     */

    this.rating = 0;
    /**
     * Server version
     */

    this.version = null;
    /**
     * Server uptime
     */

    this.uptime = null;
    /**
     * Payload loaded
     * @hidden
     */

    _ServerContext_extended.set(this, false);

    this.setupPayload();

    if (!data) {
      return;
    }

    const $ = _cheerio.default.load(data);

    if (isSearch) {
      this.payload.players = {};

      __classPrivateFieldGet(this, _ServerContext_instances, "m", _ServerContext_parseSearchTableRow).call(this, $);

      this.setupPayload();
      return;
    }

    const serverCard = extended ? __classPrivateFieldGet(this, _ServerContext_instances, "m", _ServerContext_parseServerCard).call(this, $('div.card.mb-3').filter((index, element) => {
      var _element$attribs, _element$attribs$styl;

      return element === null || element === void 0 ? void 0 : (_element$attribs = element.attribs) === null || _element$attribs === void 0 ? void 0 : (_element$attribs$styl = _element$attribs.style) === null || _element$attribs$styl === void 0 ? void 0 : _element$attribs$styl.includes('#0F0F0F');
    }).get(0), !extended) : __classPrivateFieldGet(this, _ServerContext_instances, "m", _ServerContext_parseServerCard).call(this, data, !extended);

    if (extended) {
      __classPrivateFieldSet(this, _ServerContext_extended, extended, "f");

      const [, infoCard] = $('div.row > div.col-12.col-lg-5').children().filter('div.card.mb-3').get().map(element => {
        const $ = _cheerio.default.load(element);

        return $('div.card.mb-3 > div.card-body').children();
      });
      const {
        version = null,
        uptime = null,
        country = null
      } = Object.assign.apply({}, [{}, ...infoCard.map((index, element) => {
        const $ = _cheerio.default.load(element);

        const rowValue = $('div.row > div.text-right');

        if (rowValue.children().length) {
          return {
            country: __classPrivateFieldGet(this, _ServerContext_instances, "m", _ServerContext_parseServerCountry).call(this, rowValue.children('img').get(0))
          };
        }

        const content = rowValue.contents().get(0) // @ts-ignore
        .data;
        const isUptime = content.endsWith('%');
        return {
          [isUptime ? 'uptime' : 'version']: content ? isUptime ? parseFloat(content) : content : null
        };
      }).get()]);
      this.payload = {
        version,
        country,
        uptime
      };
      this.setupPayload();
    }

    this.payload = serverCard;
    this.setupPayload();
  }
  /**
   * Check payload loaded
   */


  get isExtended() {
    return __classPrivateFieldGet(this, _ServerContext_extended, "f");
  }
  /**
   * Check like status for player
   */


  async checkLike(username) {
    const endpoint = this.options.getEndpoint({
      domain: 'api.ashcon.app'
    });
    const uuid = await (0, _utils.getUUID)(endpoint, username);
    return this.api.server.likes({
      target: this.ip,
      profile: uuid
    });
  }
  /**
   * Load all server information
   */


  async loadPayload() {
    if (this.isExtended) {
      return;
    }

    this.payload = await this.client.get(`/server/${this.ip}`).then(({
      data
    }) => new ServerContext({ ...this,
      data,
      extended: true,
      payload: {
        ip: this.ip
      }
    }));
    this.setupPayload();

    __classPrivateFieldSet(this, _ServerContext_extended, true, "f");
  }
  /**
   * @hidden
   */


  [(_ServerContext_extended = new WeakMap(), _ServerContext_instances = new WeakSet(), _ServerContext_parseServerCard = function _ServerContext_parseServerCard(data, isPreview) {
    var _bodyMotd$children$, _bodyMotd$children$$a, _bodyMotd$children$$a2, _bodyMotd$children$2;

    const $ = _cheerio.default.load(data);

    const header = $('div.card-header.p-0 > div.row.no-gutters');
    const [{
      data: title
    }] = header.find(`div.col.text-center.text-nowrap.text-ellipsis.mc-bold${isPreview ? '.p-1' : '.p-2.mc-white'}`).get(0).children;
    const [{
      data: rating
    }] = header.find(`div.col-auto.mc-gray${isPreview ? '.p-1' : '.p-2'}`).get(1).children;
    const body = $('div.card-body.p-0 > div.row.no-gutters.flex-nowrap.align-items-center');
    const {
      src: icon
    } = body.find(`div.col-auto${isPreview ? '.p-1' : '.p-2'} > img`).attr();
    const bodyMotd = body.find(`div.col.mc-reset${isPreview ? '.p-1' : '.p-2'}`).children();

    if (!((_bodyMotd$children$ = bodyMotd.children()[0]) !== null && _bodyMotd$children$ !== void 0 && (_bodyMotd$children$$a = _bodyMotd$children$.attribs) !== null && _bodyMotd$children$$a !== void 0 && (_bodyMotd$children$$a2 = _bodyMotd$children$$a.class) !== null && _bodyMotd$children$$a2 !== void 0 && _bodyMotd$children$$a2.includes('float-right')) && !((_bodyMotd$children$2 = bodyMotd.children()[0]) !== null && _bodyMotd$children$2 !== void 0 && _bodyMotd$children$2.next)) {
      throw new _error.WrapperError('SERVER_OFFLINE', title);
    }

    const motdTitle = bodyMotd.attr().title; // @ts-ignore

    const [{
      children: [{
        data: onlinePlayers
      },, {
        data: maxPlayers
      }],
      next: bodyMotdNext = {
        data: ''
      }
    }, rawMotd] = bodyMotd.children().get();
    const {
      data: textMotd
    } = bodyMotdNext;
    const motdHtml = typeof rawMotd === 'object' ? _cheerio.default.load((0, _utils.escapeColorsClasses)(rawMotd.children)).html() : textMotd;
    const motdClear = isPreview ? motdTitle : typeof rawMotd === 'object' ? (0, _utils.escapeHtml)(rawMotd.children) : textMotd;
    const parsedData = {
      icon,
      title: title,
      motd: {
        clear: motdClear,
        html: motdHtml
      },
      players: {
        online: Number(onlinePlayers),
        max: Number(maxPlayers)
      },
      rating: parseInt(rating)
    };

    if (isPreview) {
      return { ...parsedData,
        ip: __classPrivateFieldGet(this, _ServerContext_instances, "m", _ServerContext_parseIP).call(this, data),
        country: __classPrivateFieldGet(this, _ServerContext_instances, "m", _ServerContext_parseServerCountry).call(this, header.find('div.col-auto.p-1').children('img').get(0))
      };
    }

    return parsedData;
  }, _ServerContext_parseServerCountry = function _ServerContext_parseServerCountry(data) {
    if (data) {
      const {
        attribs: {
          title: name,
          src: image,
          alt: emoji
        }
      } = data;
      return {
        name,
        image,
        emoji
      };
    }

    return null;
  }, _ServerContext_parseIP = function _ServerContext_parseIP(data) {
    return data.attribs.href.replace(/\/[^]+\//, '');
  }, _ServerContext_parseSearchTableRow = function _ServerContext_parseSearchTableRow($) {
    $('td').get().forEach((element, index) => {
      const $ = _cheerio.default.load(element);

      const image = $('img');
      const row = $('td');
      const title = $('a');

      switch (index) {
        case 0:
          this.payload.icon = image.attr().src;
          break;

        case 1:
          this.payload.country = __classPrivateFieldGet(this, _ServerContext_instances, "m", _ServerContext_parseServerCountry).call(this, image.get(0));
          break;

        case 2:
          this.payload.title = title.text();
          this.payload.ip = __classPrivateFieldGet(this, _ServerContext_instances, "m", _ServerContext_parseIP).call(this, title.get(0));
          break;

        case 3:
          this.payload.motd = {
            clear: row.attr().title,
            html: ''
          };
          break;

        case 4:
          this.payload.players.online = Number(row.text());
          break;

        case 6:
          this.payload.players.max = Number(row.text());
          break;

        case 7:
          this.payload.rating = parseInt(row.text());
          break;
      }
    });
  }, _utils.kSerializeData)]() {
    return (0, _utils.pickProperties)(this, ['ip', 'title', 'icon', 'motd', 'players', 'country', 'rating', 'version', 'uptime']);
  }

}

exports.ServerContext = ServerContext;