import { Element } from 'cheerio';
import { Context, IContextOptions } from '../context';
import { RendersContext } from '../renders';
import { kSerializeData } from '../../utils';
import { Model, ModelUnion, Transformation, TransformationUnion } from './types';
export interface ISkinContextOptions extends IContextOptions<SkinContext> {
    data?: string | Element | Element[];
    isExtended?: boolean;
}
export declare class SkinContext extends Context<SkinContext> {
    #private;
    /**
     * Skin id
     */
    readonly id: string;
    /**
     * Skin name
     */
    readonly name: string;
    /**
     * Skin model
     */
    readonly model: Model | ModelUnion;
    /**
     * Skin tags
     */
    readonly tags: string[];
    /**
     * Skin rating
     */
    readonly rating: number;
    /**
     * Skin favorite count
     */
    readonly favorite: number;
    /**
     * Skin creation timestamp
     */
    readonly createdAt: number;
    /**
     * Skin transformation type
     */
    readonly transformation: Transformation | TransformationUnion | null;
    /**
     * @hidden
     */
    constructor({ data, isExtended, ...options }: ISkinContextOptions);
    /**
     * Get skin url
     */
    get url(): string;
    /**
     * Get skin renders
     */
    get renders(): RendersContext;
    /**
     * Check is slim skin model
     */
    get isSlim(): boolean;
    /**
     * Check payload loaded
     */
    get isExtended(): boolean;
    /**
     * Check is transformed
     */
    get isTransformed(): boolean;
    /**
     * Transform skin
     *
     * @see {@link https://namemc.com/skin/ee40191789e621d3 | Check "Tools" card}
     */
    transform(transformation: Transformation | TransformationUnion): Promise<SkinContext>;
    /**
     * Load all skin information
     */
    loadPayload(): Promise<void>;
    /**
     * @hidden
     */
    static parseSkinLink(link: string): Pick<SkinContext, 'id' | 'model'> | void;
    /**
     * @hidden
     */
    [kSerializeData](): any;
}
export * from './types';
