"use strict";

var _exportNames = {
  SkinContext: true
};
exports.SkinContext = void 0;

var _cheerio = require("cheerio");

var _context = require("../context");

var _renders = require("../renders");

var _error = require("../../error");

var _utils = require("../../utils");

var _types = require("./types");

Object.keys(_types).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _types[key]) return;
  exports[key] = _types[key];
});

var __classPrivateFieldGet = void 0 && (void 0).__classPrivateFieldGet || function (receiver, state, kind, f) {
  if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
  if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
  return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};

var __classPrivateFieldSet = void 0 && (void 0).__classPrivateFieldSet || function (receiver, state, value, kind, f) {
  if (kind === "m") throw new TypeError("Private method is not writable");
  if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
  if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
  return kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value), value;
};

var _SkinContext_instances, _SkinContext_extended, _SkinContext_parseSkin, _SkinContext_parseSkinCounters, _SkinContext_parseSkinTime, _SkinContext_parseSkinTags;

class SkinContext extends _context.Context {
  /**
   * @hidden
   */
  constructor({
    data,
    isExtended,
    ...options
  }) {
    super(options);

    _SkinContext_instances.add(this);
    /**
     * Skin id
     */


    this.id = '';
    /**
     * Skin name
     */

    this.name = '';
    /**
     * Skin model
     */

    this.model = _types.Model.UNKNOWN;
    /**
     * Skin tags
     */

    this.tags = [];
    /**
     * Skin rating
     */

    this.rating = 0;
    /**
     * Skin favorite count
     */

    this.favorite = 0;
    /**
     * Skin creation timestamp
     */

    this.createdAt = 0;
    /**
     * Skin transformation type
     */

    this.transformation = null;
    /**
     * Payload loaded
     * @hidden
     */

    _SkinContext_extended.set(this, false);

    this.setupPayload();

    if (!data) {
      return;
    }

    if (!isExtended) {
      const {
        id,
        model,
        name
      } = __classPrivateFieldGet(this, _SkinContext_instances, "m", _SkinContext_parseSkin).call(this, {
        data,
        isExtended
      });

      this.id = id;
      this.model = model;
      this.name = name;
    }

    const $ = _cheerio.default.load(data);

    if (isExtended) {
      __classPrivateFieldSet(this, _SkinContext_extended, true, "f");

      const {
        id,
        model
      } = __classPrivateFieldGet(this, _SkinContext_instances, "m", _SkinContext_parseSkin).call(this, {
        $,
        isExtended
      });

      this.id = id;
      this.model = model;
      this.createdAt = __classPrivateFieldGet(this, _SkinContext_instances, "m", _SkinContext_parseSkinTime).call(this, $);
      this.tags = __classPrivateFieldGet(this, _SkinContext_instances, "m", _SkinContext_parseSkinTags).call(this, $);
    }

    const counters = __classPrivateFieldGet(this, _SkinContext_instances, "m", _SkinContext_parseSkinCounters).call(this, $);

    if (counters) {
      const {
        rating,
        favorite
      } = counters;
      this.rating = rating;

      if (favorite) {
        this.favorite = favorite;
      }
    }
  }
  /**
   * Get skin url
   */


  get url() {
    return `${this.options.getEndpoint()}/texture/${this.id}.png`;
  }
  /**
   * Get skin renders
   */


  get renders() {
    const {
      options,
      client,
      api
    } = this;
    return new _renders.RendersContext({
      skin: this.id,
      model: this.model,
      options,
      client,
      api
    });
  }
  /**
   * Check is slim skin model
   */


  get isSlim() {
    return this.model === _types.Model.SLIM;
  }
  /**
   * Check payload loaded
   */


  get isExtended() {
    return __classPrivateFieldGet(this, _SkinContext_extended, "f");
  }
  /**
   * Check is transformed
   */


  get isTransformed() {
    return Boolean(this.transformation);
  }
  /**
   * Transform skin
   *
   * @see {@link https://namemc.com/skin/ee40191789e621d3 | Check "Tools" card}
   */


  transform(transformation) {
    return this.client.post('/transform-skin', `skin=${this.id}&transformation=${transformation}`, {
      headers: {
        'Content-Type': 'application/x-www-form-urlencoded',
        origin: 'https://ru.namemc.com'
      }
    }).then(({
      request
    }) => {
      var _request$res;

      const [, id] = ((request === null || request === void 0 ? void 0 : (_request$res = request.res) === null || _request$res === void 0 ? void 0 : _request$res.responseUrl) || request.responseURL).match(_utils.skinRegExp);

      if (!id) {
        throw new _error.WrapperError('NO_USEFUL');
      }

      this.payload = {
        id,
        transformation
      };
      this.setupPayload();
      return this;
    }).catch(error => {
      var _error$response;

      throw (error === null || error === void 0 ? void 0 : (_error$response = error.response) === null || _error$response === void 0 ? void 0 : _error$response.status) === 404 ? new _error.WrapperError('NOT_FOUND', this.id) : error;
    });
  }
  /**
   * Load all skin information
   */


  async loadPayload() {
    if (this.isExtended) {
      return;
    }

    this.payload = await this.client.get(`/skin/${this.id}`).then(({
      data
    }) => {
      const skin = new SkinContext({ ...this,
        data,
        isExtended: true,
        payload: {
          name: this.name
        }
      });

      if (!skin.id) {
        throw new _error.WrapperError('NO_USEFUL');
      }

      return skin;
    });
    this.setupPayload();

    __classPrivateFieldSet(this, _SkinContext_extended, true, "f");
  }
  /**
   * @hidden
   */


  static parseSkinLink(link) {
    const {
      searchParams,
      host
    } = new URL(link);
    const id = searchParams.get('id');

    if (id) {
      if (_renders.RendersContext.DOMAIN !== host) {
        _renders.RendersContext.DOMAIN = host;
      }

      return {
        id,
        model: searchParams.get('model') || _types.Model.UNKNOWN
      };
    }
  }
  /**
   * @hidden
   */


  [(_SkinContext_extended = new WeakMap(), _SkinContext_instances = new WeakSet(), _SkinContext_parseSkin = function _SkinContext_parseSkin({
    $,
    data,
    isExtended
  }) {
    if (isExtended) {
      const {
        attribs: {
          'data-id': id,
          'data-model': model
        }
      } = $('.skin-3d').get(0);
      return {
        id,
        model: model
      };
    } else {
      var _data$parent, _data$parent2;

      const id = data === null || data === void 0 ? void 0 : (_data$parent = data.parent) === null || _data$parent === void 0 ? void 0 : _data$parent.attribs.href.replace(_utils.skinRegExp, '$1');

      const $header = _cheerio.default.load(data === null || data === void 0 ? void 0 : (_data$parent2 = data.parent) === null || _data$parent2 === void 0 ? void 0 : _data$parent2.children);

      const name = $header('.card-header').text();

      const $body = _cheerio.default.load(data);

      const {
        attribs: {
          'data-src': src
        }
      } = $body('div > img.drop-shadow').get(0);
      const skin = SkinContext.parseSkinLink(src);
      return {
        id,
        name,
        model: _types.Model.UNKNOWN,
        ...skin
      };
    }
  }, _SkinContext_parseSkinCounters = function _SkinContext_parseSkinCounters($) {
    const ratingElement = $('.position-absolute.bottom-0.right-0.text-muted').get(0);
    const counters = ratingElement === null || ratingElement === void 0 ? void 0 : ratingElement.children;

    if (!ratingElement || !counters) {
      return;
    }

    if (counters) {
      if (counters.length > 1) {
        const [{
          data: favoriteCount
        },, {
          data: rating
        }] = ratingElement;
        return {
          rating: parseInt(rating),
          favorite: parseInt(favoriteCount)
        };
      } else {
        // @ts-ignore invalid lib types
        const {
          children: [{
            data: rating
          }]
        } = ratingElement;
        return {
          rating: parseInt(rating)
        };
      }
    }

    return {
      rating: parseInt(ratingElement === null || ratingElement === void 0 ? void 0 : ratingElement.data)
    };
  }, _SkinContext_parseSkinTime = function _SkinContext_parseSkinTime($) {
    const {
      attribs: {
        title
      }
    } = $('.position-absolute.bottom-0.left-0.text-muted.title-time').get(0);
    return (0, _utils.convertDate)(title);
  }, _SkinContext_parseSkinTags = function _SkinContext_parseSkinTags($) {
    return $('div.card-body.text-center.py-1 > a.badge.badge-pill').map((index, element) => {
      // @ts-ignore
      const {
        children: [{
          data: tag
        }]
      } = element;
      return tag;
    }).get();
  }, _utils.kSerializeData)]() {
    return (0, _utils.pickProperties)(this, ['id', 'name', 'url', 'model', 'isSlim', 'transformation', 'isTransformed', 'renders', 'tags', 'rating', 'favorite', 'createdAt']);
  }

}

exports.SkinContext = SkinContext;