"use strict";

exports.WrapperError = exports.ErrorDescription = void 0;
var ErrorDescription;
exports.ErrorDescription = ErrorDescription;

(function (ErrorDescription) {
  ErrorDescription["UNKNOWN"] = "Unknown error.";
  ErrorDescription["CLOUDFLARE"] = "NameMC uses CloudFlare, which blocks requests from bots. Use CloudProxy. Check out the README for more information.";
  ErrorDescription["INVALID_NICKNAME"] = "$0 is invalid Minecraft nickname.";
  ErrorDescription["NOT_FOUND"] = "Resource not found.";
  ErrorDescription["NO_USEFUL"] = "There is no useful information in the data received, try later or report this to the Github issue.";
  ErrorDescription["SERVER_OFFLINE"] = "Server $0 offline.";
})(ErrorDescription || (exports.ErrorDescription = ErrorDescription = {}));
/**
 * @hidden
 */


class WrapperError extends Error {
  constructor(error, params = []) {
    if (!Array.isArray(params)) {
      params = [params];
    }

    const code = error;
    let description = ErrorDescription[error];
    params.forEach((param, index) => description = description.replace(`$${index}`, param));
    super(description);
    this.code = code;
    this.name = this.constructor.name;

    if (Error.captureStackTrace) {
      Error.captureStackTrace(this, this.constructor);
    } else {
      this.stack = new Error().stack;
    }
  }

  get [Symbol.toStringTag]() {
    return this.constructor.name;
  }

  toJSON() {
    return this;
  }

}

exports.WrapperError = WrapperError;