"use strict";

var _exportNames = {
  Tab: true,
  Section: true,
  NameMC: true
};
exports.Tab = exports.Section = exports.NameMC = void 0;

var _axios = require("axios");

var _api = require("./api");

Object.keys(_api).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _api[key]) return;
  exports[key] = _api[key];
});

var _options = require("./options");

Object.keys(_options).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _options[key]) return;
  exports[key] = _options[key];
});

var _parser = require("./parser");

var _error = require("./error");

var _contexts = require("./contexts");

Object.keys(_contexts).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _contexts[key]) return;
  exports[key] = _contexts[key];
});

var _utils = require("./utils");

Object.keys(_utils).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _utils[key]) return;
  exports[key] = _utils[key];
});

var __classPrivateFieldGet = void 0 && (void 0).__classPrivateFieldGet || function (receiver, state, kind, f) {
  if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
  if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
  return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};

var _NameMC_instances, _NameMC_addInterceptors;

const DESKTOP_USER_AGENT = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36';
var Tab;
exports.Tab = Tab;

(function (Tab) {
  Tab["TRENDING"] = "trending";
  Tab["NEW"] = "new";
  Tab["RANDOM"] = "random";
  Tab["TAG"] = "tag";
})(Tab || (exports.Tab = Tab = {}));

var Section;
exports.Section = Section;

(function (Section) {
  Section["DAILY"] = "daily";
  Section["WEEKLY"] = "weekly";
  Section["MONTHLY"] = "monthly";
  Section["TOP"] = "top";
})(Section || (exports.Section = Section = {}));

class NameMC extends _parser.DataParser {
  constructor(options = {}) {
    super();

    _NameMC_instances.add(this);

    this.options = new _options.Options(options);
    this.api = new _api.API();
    this.client = _axios.default.create({
      baseURL: this.options.getEndpoint(),
      headers: typeof window === 'undefined' ? {
        'User-Agent': DESKTOP_USER_AGENT
      } : {}
    });

    __classPrivateFieldGet(this, _NameMC_instances, "m", _NameMC_addInterceptors).call(this);
  }
  /**
   * Get skin history by nickname
   */


  async skinHistory({
    username,
    page = 1
  }) {
    const {
      uuid
    } = await this.getPlayer(username);
    return this.client.get(`/minecraft-skins/profile/${uuid}`, {
      params: {
        page
      }
    }).then(({
      data
    }) => {
      var _skins$;

      const skins = this.parseSkins(data);

      if ((_skins$ = skins[0]) !== null && _skins$ !== void 0 && _skins$.id) {
        return skins;
      }

      throw new _error.WrapperError('NO_USEFUL');
    });
  }
  /**
   * Get player info by nickname
   */


  async getPlayer(username) {
    const context = new _contexts.PlayerContext({
      payload: {
        username
      },
      ...this
    });
    await context.loadPayload();
    return context;
  }
  /**
   * Get skin renders
   */


  getRenders(options) {
    return new _contexts.RendersContext({ ...this,
      ...options
    });
  }
  /**
   * Transform skin method
   */


  transformSkin({
    skin,
    transformation = _contexts.Transformation.GRAYSCALE,
    model = _contexts.Model.CLASSIC
  }) {
    return new _contexts.SkinContext({ ...this,
      payload: {
        id: skin,
        model
      }
    }).transform(transformation);
  }
  /**
   * Get cape info by cape hash
   */


  getCapeInfo(hash) {
    return new _contexts.CapeContext({
      hash,
      ...this
    });
  }
  /**
   * Get player friends by nickname
   */


  async getFriends(username) {
    const endpoint = this.options.getEndpoint({
      domain: 'api.ashcon.app'
    });
    const uuid = await (0, _utils.getUUID)(endpoint, username);
    return this.api.profile.friends({
      target: uuid
    });
  }
  /**
   * Get skins from a specific tab of the site
   */


  getSkins({
    tab,
    section = Section.WEEKLY,
    page = 1
  }) {
    return this.client.get(`/minecraft-skins/${tab}${tab === 'trending' || tab === 'tag' && section !== 'weekly' ? `/${section}` : ''}`, {
      params: {
        page
      }
    }).then(({
      data
    }) => {
      const skins = this.parseSkins(data);

      if (!skins.length) {
        throw new _error.WrapperError('NO_USEFUL');
      }

      return skins;
    });
  }
  /**
   * Get information about skin
   */


  async getSkin(id) {
    const skin = new _contexts.SkinContext({ ...this,
      payload: {
        id
      }
    });
    await skin.loadPayload();
    return skin;
  }
  /**
   * Get servers list
   */


  getServers(page = 1) {
    return this.client.get(`/minecraft-servers/${page}`).then(({
      data
    }) => {
      var _servers$;

      const servers = this.parseServers(data);

      if (!((_servers$ = servers[0]) !== null && _servers$ !== void 0 && _servers$.ip)) {
        throw new _error.WrapperError('NO_USEFUL');
      }

      return servers;
    });
  }
  /**
   * Get server by ip
   */


  async getServer(ip) {
    const server = new _contexts.ServerContext({ ...this,
      payload: {
        ip
      }
    });
    await server.loadPayload();
    return server;
  }
  /**
   * Get server likes by ip
   */


  getServerLikes(ip) {
    return this.api.server.likes({
      target: ip
    });
  }
  /**
   * Check server like value by username
   */


  checkServerLike({
    ip,
    username
  }) {
    return new _contexts.ServerContext({ ...this,
      payload: {
        ip
      }
    }).checkLike(username);
  }
  /**
   * Search
   */


  search(query) {
    return this.client.get('/search', {
      params: {
        q: query
      }
    }).then(({
      request,
      headers,
      data
    }) => {
      var _request$res;

      const finalUrl = headers['x-final-url'] || (request === null || request === void 0 ? void 0 : (_request$res = request.res) === null || _request$res === void 0 ? void 0 : _request$res.responseUrl) || request.responseURL;

      if (_utils.profileRegExp.test(finalUrl)) {
        return new _contexts.PlayerContext({ ...this,
          data
        });
      }

      if (_utils.serverRegExp.test(finalUrl)) {
        return new _contexts.ServerContext({ ...this,
          data,
          extended: true
        });
      }

      return new _contexts.SearchContext({ ...this,
        data,
        payload: {
          query
        }
      });
    });
  }

}

exports.NameMC = NameMC;
_NameMC_instances = new WeakSet(), _NameMC_addInterceptors = function _NameMC_addInterceptors() {
  const {
    proxy,
    cloudProxy
  } = this.options;

  if (proxy && cloudProxy) {
    this.client.interceptors.request.use(config => {
      const {
        url,
        baseURL,
        method,
        data,
        params
      } = config;
      delete config.baseURL;
      delete config.params;
      const searchParams = params ? new URLSearchParams(Object.entries(params)).toString() : null;
      config.url = proxy;
      config.method = 'post';
      config.data = { ...cloudProxy,
        url: `${baseURL}${url}${searchParams ? `?${searchParams}` : ''}`,
        cmd: `request.${method}`,
        headers: config.headers,
        postData: method === 'post' ? data : undefined
      };
      return config;
    });
  }

  this.client.interceptors.response.use(response => {
    if (proxy && cloudProxy) {
      const {
        data: {
          solution: {
            url,
            response: htmlPage
          }
        }
      } = response;
      response.request.res.responseUrl = url;
      response.data = htmlPage;
    }

    return response;
  }, error => {
    var _error$response;

    switch (error === null || error === void 0 ? void 0 : (_error$response = error.response) === null || _error$response === void 0 ? void 0 : _error$response.status) {
      case 403:
        throw new _error.WrapperError('CLOUDFLARE');

      case 404:
        throw new _error.WrapperError('NOT_FOUND');

      default:
        throw error;
    }
  });
};